@@ -15,21 +15,23 @@
 #      along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import abc
-import datetime
+from datetime import datetime, timezone
 from enum import Enum
 from typing import List, Dict, Any, Tuple
 
 
-class Municipality:
-    def __init__(self, /, **kwargs):
-        self.id = kwargs.get("id")
-        self.name = kwargs.get("name")
+class Area:
+    def __init__(self, /, **kwargs) -> object:
+        self.id: int = int(kwargs.get("id", 0))
+        self.name: str = kwargs.get("name", "")
+        self.municipality: str = kwargs.get("municipality", "")
+        self.province: Province = Province(kwargs.get("province", 0))
 
     def __str__(self):
         return f"{self.name}"
 
     def __repr__(self):
-        return f"Municipality({self.id}, {self.name}"
+        return f"Area({self.id}, {self.name}, {self.municipality}, {self.province})"
 
 
 class Province(Enum):
@@ -44,6 +46,20 @@ class Province(Enum):
     NORTERN_CAPE = 8
     WESTERN_CAPE = 9
 
+    @staticmethod
+    def from_name(name: str):
+        return {
+            "Eastern Cape": Province.EASTERN_CAPE,
+            "Free State": Province.FREE_STATE,
+            "Gauteng": Province.GAUTENG,
+            "Kwa-Zulu Natal": Province.KWAZULU_NATAL,
+            "Limpopo": Province.LIMPOPO,
+            "Mpumalanga": Province.MPUMALANGA,
+            "North West": Province.NORTH_WEST,
+            "Nortern Cape": Province.NORTERN_CAPE,
+            "Western Cape": Province.WESTERN_CAPE,
+        }.get(name, Province.UNKNOWN)
+
     def __str__(self):
         return {
             self.EASTERN_CAPE: "Eastern Cape",
@@ -55,7 +71,7 @@ class Province(Enum):
             self.NORTH_WEST: "North West",
             self.NORTERN_CAPE: "Nortern Cape",
             self.WESTERN_CAPE: "Western Cape",
-        }.get(self, "Unknown")
+        }.get(self, "Unknown Province")
 
 
 class StageError(Exception):
@@ -76,36 +92,19 @@ class Stage(Enum):
 
     def __str__(self):
         return {
-            self.NO_LOAD_SHEDDING.value: "No Load Shedding",
-            self.STAGE_1.value: "Stage 1",
-            self.STAGE_2.value: "Stage 2",
-            self.STAGE_3.value: "Stage 3",
-            self.STAGE_4.value: "Stage 4",
-            self.STAGE_5.value: "Stage 5",
-            self.STAGE_6.value: "Stage 6",
-            self.STAGE_7.value: "Stage 7",
-            self.STAGE_8.value: "Stage 8",
-        }.get(self.value, "Unknown")
-
-
-class Suburb:
-    def __init__(self, /, **kwargs):
-        self.id = kwargs.get("id")
-        self.name = kwargs.get("name")
-        self.municipality = kwargs.get("municipality")
-        self.province = Provider.province_from_name(kwargs.get("province"))
-        self.total = kwargs.get("total")
-
-    def __str__(self):
-        return f"{self.name}"
+            self.NO_LOAD_SHEDDING: "No Load Shedding",
+            self.STAGE_1: "Stage 1",
+            self.STAGE_2: "Stage 2",
+            self.STAGE_3: "Stage 3",
+            self.STAGE_4: "Stage 4",
+            self.STAGE_5: "Stage 5",
+            self.STAGE_6: "Stage 6",
+            self.STAGE_7: "Stage 7",
+            self.STAGE_8: "Stage 8",
+        }.get(self, "Unknown Stage")
 
     def __repr__(self):
-        return f"Suburb({self.id}, {self.name}, {self.municipality}, {self.province}, {self.total})"
-
-
-class Timeslot:
-    start: datetime.date
-    end: datetime.date
+        return self.__str__()
 
 
 def dict_list_to_obj_list(data: List[Dict], T: Any) -> List[Any]:
@@ -119,18 +118,35 @@ def dict_list_to_obj_list(data: List[Dict], T: Any) -> List[Any]:
 def obj_list_to_dict_list(data: List[Any]) -> List[Dict]:
     dicts: List[Dict] = []
     for d in data:
-        # s = T(**d)
         dicts.append(d.__dict__)
     return dicts
 
 
-def filter_empty_suburbs(suburbs: List[Suburb]) -> List[Suburb]:
-    filtered: List[Suburb] = []
-    for s in suburbs:
-        if not s.total:
-            continue
-        filtered.append(s)
-    return filtered
+def to_utc(data: List[Tuple]) -> List[Tuple]:
+    out: List[Tuple] = []
+    for d in data:
+        start = d[0].astimezone(timezone.utc)
+        end = d[1].astimezone(timezone.utc)
+        out.append((start, end))
+    return out
+
+
+def datetime_to_isoformat(data: List[Tuple]) -> List[Tuple]:
+    out: List[Tuple] = []
+    for d in data:
+        start = d[0].astimezone(timezone.utc).isoformat()
+        end = d[1].astimezone(timezone.utc).isoformat()
+        out.append((start, end))
+    return out
+
+
+def isoformat_to_datetime(data: List[Tuple]) -> List[Tuple]:
+    out: List[Tuple] = []
+    for d in data:
+        start = datetime.fromisoformat(d[0]).astimezone(timezone.utc)
+        end = datetime.fromisoformat(d[1]).astimezone(timezone.utc)
+        out.append((start, end))
+    return out
 
 
 class ProviderError(Exception):
@@ -138,31 +154,15 @@ class ProviderError(Exception):
 
 
 class Provider(abc.ABC):
-    name: str = ""
-
-    @classmethod
-    def province_from_name(cls, name: str) -> Province:
-        return {
-            "Eastern Cape": Province.EASTERN_CAPE,
-            "Free State": Province.FREE_STATE,
-            "Gauteng": Province.GAUTENG,
-            "Kwa-Zulu Natal": Province.KWAZULU_NATAL,
-            "Limpopo": Province.LIMPOPO,
-            "Mpumalanga": Province.MPUMALANGA,
-            "North West": Province.NORTH_WEST,
-            "Nortern Cape": Province.NORTERN_CAPE,
-            "Western Cape": Province.WESTERN_CAPE,
-        }.get(name, Province.UNKNOWN)
+    name: str = "Provider"
 
     @abc.abstractmethod
-    def get_stage(self) -> Stage:
-        pass
-
-    @abc.abstractmethod
-    def get_schedule(self) -> List[Tuple]:
+    def get_areas(self, search_text: str = None, max_results: int = None) -> List[Area]:
         raise NotImplemented
 
     @abc.abstractmethod
-    def search(self, search_text: str, max_results: int = None) -> List[Any]:
-        pass
+    def get_schedule(self, area: Area, stage: Stage) -> List[Tuple]:
+        raise NotImplemented
 
+    def get_stage(self) -> Stage:
+        pass
